@@ -166,7 +166,6 @@ module Text.Pandoc.Builder ( module Text.Pandoc.Definition
 where
 import Text.Pandoc.Definition
 import Data.String
-import Data.Monoid
 import qualified Data.Map as M
 import Data.Sequence (Seq, (|>), viewr, viewl, ViewR(..), ViewL(..))
 import qualified Data.Sequence as Seq
@@ -177,6 +176,7 @@ import Data.List (groupBy)
 import Data.Data
 import Control.Arrow ((***))
 import GHC.Generics (Generic)
+import Data.Semigroup
 
 #if MIN_VERSION_base(4,5,0)
 -- (<>) is defined in Data.Monoid
@@ -209,15 +209,15 @@ isNull = Seq.null . unMany
 type Inlines = Many Inline
 type Blocks  = Many Block
 
+deriving instance Semigroup Blocks
 deriving instance Monoid Blocks
 
-instance Monoid Inlines where
-  mempty = Many mempty
-  (Many xs) `mappend` (Many ys) =
+instance Semigroup Inlines where
+  (Many xs) <> (Many ys) =
     case (viewr xs, viewl ys) of
       (EmptyR, _) -> Many ys
       (_, EmptyL) -> Many xs
-      (xs' :> x, y :< ys') -> Many (meld `mappend` ys')
+      (xs' :> x, y :< ys') -> Many (meld <> ys')
         where meld = case (x, y) of
                           (Space, Space)     -> xs' |> Space
                           (Space, SoftBreak) -> xs' |> SoftBreak
@@ -234,6 +234,9 @@ instance Monoid Inlines where
                           (LineBreak, SoftBreak) -> xs' |> LineBreak
                           (SoftBreak, SoftBreak) -> xs' |> SoftBreak
                           _                  -> xs' |> x |> y
+instance Monoid Inlines where
+  mempty = Many mempty
+  mappend = (<>)
 
 instance IsString Inlines where
    fromString = text
@@ -478,19 +481,25 @@ table :: Inlines               -- ^ Caption
       -> Blocks
 table caption cellspecs headers rows = singleton $
   Table (toList caption) aligns widths
-      (map toList headers) (map (map toList) rows)
+      (map toList headers') (map (map toList) rows)
    where (aligns, widths) = unzip cellspecs
+         numcols  = case (headers:rows) of
+                         [] -> 0
+                         xs -> maximum (map length xs)
+         headers' = if null headers
+                       then replicate numcols mempty
+                       else headers
 
 -- | A simple table without a caption.
 simpleTable :: [Blocks]   -- ^ Headers
             -> [[Blocks]] -- ^ Rows
             -> Blocks
-simpleTable headers = table mempty (mapConst defaults headers) headers
+simpleTable headers rows =
+  table mempty (replicate numcols defaults) headers rows
   where defaults = (AlignDefault, 0)
+        numcols  = case (headers:rows) of
+                        [] -> 0
+                        xs -> maximum (map length xs)
 
 divWith :: Attr -> Blocks -> Blocks
 divWith attr = singleton . Div attr . toList
-
-mapConst :: Functor f => b -> f a -> f b
-mapConst = fmap . const
-
